# Generated by Django 4.0.3 on 2022-03-30 20:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ipam', '0057_created_datetimefield'),
        ('circuits', '0034_created_datetimefield'),
    ]

    operations = [
        migrations.AddField(
            model_name='provider',
            name='asns',
            field=models.ManyToManyField(blank=True, related_name='providers', to='ipam.asn'),
        ),
    ]
