# Generated by Django 3.2.10 on 2022-01-08 18:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0150_interface_vrf'),
    ]

    operations = [
        migrations.AddField(
            model_name='interface',
            name='duplex',
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name='interface',
            name='speed',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
