# Webhook content types
HTTP_CONTENT_TYPE_JSON = 'application/json'

# Registerable extras features
EXTRAS_FEATURES = [
    'custom_fields',
    'custom_links',
    'export_templates',
    'job_results',
    'journaling',
    'tags',
    'webhooks'
]
