# Generated by Django 3.2.11 on 2022-01-10 16:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0069_custom_object_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='customlink',
            name='enabled',
            field=models.BooleanField(default=True),
        ),
    ]
